% Given an image with ellipse state, generates a color spatiogram of the pixels
% inside the ellipse, then randomly samples the cdf of the spatiogram to
% generate a new image with approximately the same spatiogram.  Used for
% some figures in the paper.
function newim = generate_image_from_spatiogram_of_image(im, state, aspect_ratio, nbins)

    % create perimeter and mask for ellipse
    [ellipse_perimeter, ellipse_mask] = generate_ellipse(size(im), state, aspect_ratio)
%     ellipse_perimeter = ellipse(state, aspect_ratio);
%     ellipse_mask = create_ellipse_binary_mask(im, ellipse_perimeter);
    
    % compute color spatiogram; vectorize and normalize to make PDF
    spat = compute_color_spatiogram(im, state, aspect_ratio, nbins);
    spat_n = max(spat.histogram, 1);  % prevents divide by zero; won't affect 
                                        % results b/c if histogram is zero then pdf will be zero, 
                                        % so mean and var won't matter
    spat_meanx = spat.sumx ./ spat_n;
    spat_meany = spat.sumy ./ spat_n;
    spat_varx = spat.sumx2 ./ spat_n - spat_meanx.^2;
    spat_vary = spat.sumy2 ./ spat_n - spat_meany.^2;
    
    % prevents divide by zero; necessary in two cases:  (1) when there are
    % no pixels in this bin of the histogram, in which case it doesn't
    % matter what value we choose; (2) when all the pixels in the bin are
    % in the same location, in which case we need a default sigma (sigma=1
    % seems like a good choice).
    spat_varx = max(spat_varx, 1);
    spat_vary = max(spat_vary, 1);

    % generate new image using histogram as generative model
    % (i.e., by randomly sampling cdf of histogram)
    newim = zeros(size(im));
    for i=1:size(newim,1),
        for j=1:size(newim,2),

            % generate pdf of spatiogram
            hpdf = spat.histogram .* exp(- ((j - spat_meanx).^2 ./ spat_varx + (i - spat_meany).^2 ./ spat_vary));
            hpdf = hpdf(:);
            hpdf = hpdf / sum(hpdf);
            
            % generate cdf of spatiogram
            hcdf = zeros(size(hpdf));
            hcdf(1) = hpdf(1);  
            for k=2:length(hpdf),
                hcdf(k) = hcdf(k-1) + hpdf(k);
            end
            
            % find bin(s) closest to a random sampling of cdf
            diff = abs(hcdf-rand(1));
            ii = find(diff == min(diff));
            indx = ii(1);
            % get color of 'indx' bin in the histogram
            indx3 = floor((indx-1) / (nbins(1)*nbins(2))) + 1;
            leftover = mod((indx-1), nbins(1)*nbins(2));
            indx2 = floor(leftover / nbins(1)) + 1;
            indx1 = mod(leftover, nbins(1)) + 1;
			binwidth1 = 256/nbins(1);
			binwidth2 = 256/nbins(2);
			binwidth3 = 256/nbins(3);
            color1 = binwidth1 * ( indx1 - 0.5);
            color2 = binwidth2 * ( indx2 - 0.5);
            color3 = binwidth3 * ( indx3 - 0.5);
            if (color1>255 | color2>255 | color3>255), error('assert'), end
            newim(i,j,1) = color1;
            newim(i,j,2) = color2;
            newim(i,j,3) = color3;
        end
    end

    for i=1:3,
        newim(:,:,i) = newim(:,:,i) .* ellipse_mask;        
    end

